package org.antlr.analysis;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import org.antlr.analysis.DFA;
import org.antlr.analysis.SemanticContext;
import org.antlr.codegen.CodeGenerator;
import org.antlr.tool.Grammar;
import org.antlr.tool.GrammarAST;
import org.stringtemplate.v4.ST;
import org.stringtemplate.v4.STGroup;

public class Predicate extends SemanticContext {
  public GrammarAST predicateAST;
  
  protected boolean gated = false;
  
  protected boolean synpred = false;
  
  public static final int INVALID_PRED_VALUE = -2;
  
  public static final int FALSE_PRED = 0;
  
  public static final int TRUE_PRED = -1;
  
  protected int constantValue = -2;
  
  public Predicate(int constantValue) {
    this.predicateAST = new GrammarAST();
    this.constantValue = constantValue;
  }
  
  public Predicate(GrammarAST predicate) {
    this.predicateAST = predicate;
    this.gated = (predicate.getType() == 41 || predicate.getType() == 90);
    this.synpred = (predicate.getType() == 90 || predicate.getType() == 14);
  }
  
  public Predicate(Predicate p) {
    this.predicateAST = p.predicateAST;
    this.gated = p.gated;
    this.synpred = p.synpred;
    this.constantValue = p.constantValue;
  }
  
  public boolean equals(Object o) {
    if (!(o instanceof Predicate))
      return false; 
    Predicate other = (Predicate)o;
    if (this.constantValue != other.constantValue)
      return false; 
    if (this.constantValue != -2)
      return true; 
    return this.predicateAST.getText().equals(other.predicateAST.getText());
  }
  
  public int hashCode() {
    if (this.constantValue != -2)
      return this.constantValue; 
    if (this.predicateAST == null)
      return 0; 
    return this.predicateAST.getText().hashCode();
  }
  
  public ST genExpr(CodeGenerator generator, STGroup templates, DFA dfa) {
    ST eST;
    if (templates != null) {
      if (this.synpred) {
        eST = templates.getInstanceOf("evalSynPredicate");
      } else {
        eST = templates.getInstanceOf("evalPredicate");
        generator.grammar.decisionsWhoseDFAsUsesSemPreds.add(dfa);
      } 
      String predEnclosingRuleName = this.predicateAST.enclosingRuleName;
      if (generator != null)
        eST.add("pred", generator.translateAction(predEnclosingRuleName, this.predicateAST)); 
    } else {
      eST = new ST("<pred>");
      eST.add("pred", toString());
      return eST;
    } 
    if (generator != null) {
      String description = generator.target.getTargetStringLiteralFromString(toString());
      eST.add("description", description);
    } 
    return eST;
  }
  
  public SemanticContext getGatedPredicateContext() {
    if (this.gated)
      return this; 
    return null;
  }
  
  public boolean hasUserSemanticPredicate() {
    return (this.predicateAST != null && (this.predicateAST.getType() == 41 || this.predicateAST.getType() == 83));
  }
  
  public boolean isSyntacticPredicate() {
    return (this.predicateAST != null && (this.predicateAST.getType() == 90 || this.predicateAST.getType() == 14));
  }
  
  public void trackUseOfSyntacticPredicates(Grammar g) {
    if (this.synpred)
      g.synPredNamesUsedInDFA.add(this.predicateAST.getText()); 
  }
  
  public String toString() {
    if (this.predicateAST == null)
      return "<nopred>"; 
    return this.predicateAST.getText();
  }
}
